//////////////////////////////////////////////////////////////////////////
//
//  This source file is part of StampEngine Virtual Globe
//  Copyright (c) 2008-2009 by The StampEngine Team . All Rights Reserved
//
//////////////////////////////////////////////////////////////////////////
//
//  Filename: se_vector4.inl
//  Author  : StampEngine Team 
//  Modifier: StampEngine Team  
//  Created : 
//  Purpose : vector4 template
//	Reference : http://www.geometrictools.com ( Wild Magic Library )
//				http://www.ogre3d.org/		  ( Ogre Object-Oriented Graphics Rendering Engine )
//				http://mathworld.wolfram.com/Matrix.html
//
//////////////////////////////////////////////////////////////////////////
#ifndef _SE_VECTOR4_INL_
#define _SE_VECTOR4_INL_
#include"matht_tool.h"
template <class real>
vector4_t<real>::vector4_t ()
{

}

template <class real>
vector4_t<real>::vector4_t (real fx, real fy, real fz, real fw)
{
	m_tuple[0] = fx;
	m_tuple[1] = fy;
	m_tuple[2] = fz;
	m_tuple[3] = fw;
}


template <class real>
vector4_t<real>::vector4_t (const vector4_t& v)
{
	m_tuple[0] = v.m_tuple[0];
	m_tuple[1] = v.m_tuple[1];
	m_tuple[2] = v.m_tuple[2];
	m_tuple[3] = v.m_tuple[3];
}


template <class real>
vector4_t<real>& vector4_t<real>::operator= (const vector4_t& v)
{
	m_tuple[0] = v.m_tuple[0];
	m_tuple[1] = v.m_tuple[1];
	m_tuple[2] = v.m_tuple[2];
	m_tuple[3] = v.m_tuple[3];
	return *this;
}



//
// compare
//
template <class real>
bool vector4_t<real>::operator== (const vector4_t& v) const
{
	real diff_x = m_tuple[0] - v.m_tuple[0];
	real diff_y = m_tuple[1] - v.m_tuple[1];
	real diff_z = m_tuple[2] - v.m_tuple[2];
	real diff_w = m_tuple[3] - v.m_tuple[3];

	return	(diff_x <= math_tool<real>::s_zero_tolerance ) &&
		(diff_y <= math_tool<real>::s_zero_tolerance) &&
		(diff_z <= math_tool<real>::s_zero_tolerance) &&
		(diff_w <= math_tool<real>::s_zero_tolerance);
}

template <class real>
bool vector4_t<real>::operator!= (const vector4_t& v) const
{
	real diff_x = m_tuple[0] - v.m_tuple[0];
	real diff_y = m_tuple[1] - v.m_tuple[1];
	real diff_z = m_tuple[2] - v.m_tuple[2];
	real diff_w = m_tuple[3] - v.m_tuple[3];

	return	(diff_x > math_tool<real>::s_zero_tolerance) ||
		(diff_y > math_tool<real>::s_zero_tolerance) ||
		(diff_z > math_tool<real>::s_zero_tolerance) ||
		(diff_w > math_tool<real>::s_zero_tolerance);
}


//
// arithmetic
//
template <class real>
vector4_t<real> vector4_t<real>::operator+ (const vector4_t& v) const
{
	return vector4_t(
		m_tuple[0] + v.m_tuple[0],
		m_tuple[1] + v.m_tuple[1],
		m_tuple[2] + v.m_tuple[2],
		m_tuple[3] + v.m_tuple[3]);
}

template <class real>
vector4_t<real> vector4_t<real>::operator- (const vector4_t& v) const
{
	return vector4_t(
		m_tuple[0] - v.m_tuple[0],
		m_tuple[1] - v.m_tuple[1],
		m_tuple[2] - v.m_tuple[2],
		m_tuple[3] - v.m_tuple[3]);
}

template <class real>
vector4_t<real> vector4_t<real>::operator* (real scalar) const
{
	return vector4_t(
		scalar * m_tuple[0],
		scalar * m_tuple[1],
		scalar * m_tuple[2],
		scalar * m_tuple[3]);
}

template <class real>
vector4_t<real> vector4_t<real>::operator/ (real scalar) const
{
	vector4_t quot;

	if ( scalar != (real)0.0 )
	{
		real inv_scalar = ((real)1.0)/scalar;
		quot.m_tuple[0] = inv_scalar * m_tuple[0];
		quot.m_tuple[1] = inv_scalar * m_tuple[1];
		quot.m_tuple[2] = inv_scalar * m_tuple[2];
		quot.m_tuple[3] = inv_scalar * m_tuple[3];
	}
	else
	{
		quot.m_tuple[0] = math_tool<real>::s_max_real;
		quot.m_tuple[1] = math_tool<real>::s_max_real;
		quot.m_tuple[2] = math_tool<real>::s_max_real;
		quot.m_tuple[3] = math_tool<real>::s_max_real;
	}

	return quot;
}

template <class real>
vector4_t<real> vector4_t<real>::operator- () const
{
	return vector4_t(
		-m_tuple[0],
		-m_tuple[1],
		-m_tuple[2],
		-m_tuple[3]);
}

template <class real>
vector4_t<real> operator* (real scalar, const vector4_t<real>& v)
{
	return vector4_t<real>(
		scalar * v[0],
		scalar * v[1],
		scalar * v[2],
		scalar * v[3]);
}

template <class real>
vector4_t<real>& vector4_t<real>::operator+= (const vector4_t& v)
{
	m_tuple[0] += v.m_tuple[0];
	m_tuple[1] += v.m_tuple[1];
	m_tuple[2] += v.m_tuple[2];
	m_tuple[3] += v.m_tuple[3];
	return *this;
}

template <class real>
vector4_t<real>& vector4_t<real>::operator-= (const vector4_t& v)
{
	m_tuple[0] -= v.m_tuple[0];
	m_tuple[1] -= v.m_tuple[1];
	m_tuple[2] -= v.m_tuple[2];
	m_tuple[3] -= v.m_tuple[3];
	return *this;
}

template <class real>
vector4_t<real>& vector4_t<real>::operator*= (real scalar)
{
	m_tuple[0] *= scalar;
	m_tuple[1] *= scalar;
	m_tuple[2] *= scalar;
	m_tuple[3] *= scalar;
	return *this;
}

template <class real>
vector4_t<real>& vector4_t<real>::operator/= (real scalar)
{
	if ( scalar != (real)0.0 )
	{
		real inv_scalar = ((real)1.0)/scalar;
		m_tuple[0] *= inv_scalar;
		m_tuple[1] *= inv_scalar;
		m_tuple[2] *= inv_scalar;
		m_tuple[3] *= inv_scalar;
	}
	else
	{
		m_tuple[0] = math_tool<real>::s_max_real;
		m_tuple[1] = math_tool<real>::s_max_real;
		m_tuple[2] = math_tool<real>::s_max_real;
		m_tuple[3] = math_tool<real>::s_max_real;
	}

	return *this;
}

template <class real>
real vector4_t<real>::length () const
{
	return math_tool<real>::sqrt_(
		m_tuple[0]*m_tuple[0] +
		m_tuple[1]*m_tuple[1] +
		m_tuple[2]*m_tuple[2] +
		m_tuple[3]*m_tuple[3]);
}
//----------------------------------------------------------------------------
template <class real>
real vector4_t<real>::squared_length () const
{
	return
		m_tuple[0]*m_tuple[0] +
		m_tuple[1]*m_tuple[1] +
		m_tuple[2]*m_tuple[2] +
		m_tuple[3]*m_tuple[3];
}
//----------------------------------------------------------------------------
template <class real>
real vector4_t<real>::dot (const vector4_t& v) const
{
	return
		m_tuple[0]*v.m_tuple[0] +
		m_tuple[1]*v.m_tuple[1] +
		m_tuple[2]*v.m_tuple[2] +
		m_tuple[3]*v.m_tuple[3];
}
//----------------------------------------------------------------------------
template <class real>
real vector4_t<real>::normalize ()
{
	real length_val = length();

	if ( length_val > math_tool<real>::s_zero_tolerance )
	{
		real inv_length = ((real)1.0)/length_val;
		m_tuple[0] *= inv_length;
		m_tuple[1] *= inv_length;
		m_tuple[2] *= inv_length;
		m_tuple[3] *= inv_length;
	}
	else
	{
		length_val = (real)0.0;
		m_tuple[0] = (real)0.0;
		m_tuple[1] = (real)0.0;
		m_tuple[2] = (real)0.0;
		m_tuple[3] = (real)0.0;
	}

	return length_val;
}


#endif //_SE_VECTOR4_INL_

